#ifndef VCF_FORMAT_H
#define VCF_FORMAT_H

#include <stdint.h>
#include <kmacros.h>

/* NOTE: strings for A and B allele for SNPs and single base in/dels will
   point to fixed strings above. For multiple base polymorphisms, 
   strings are allocated */
typedef struct {
  uint32_t pos;
  int8_t q;
  int8_t flags;
  uint8_t a_int;
  uint8_t b_int;
  char *a_allele;
  char *b_allele;

  int nAA;
  int nAB;
  int nBB;
  int n_obs;
  int n_missing;

  double a_freq;
  double b_freq;
  
} snp_t;

typedef struct {
  char *sample_name;
  int subpop_index;
  int column_index;
} sample_t;

typedef struct {
  char *chm_name;

  int n_snps;
  snp_t *snps;
  int8_t **alleles; 
} chm_t;

typedef struct {
  FILE *f;
  char *vcf_fname;

  int n_samples;
  sample_t *samples;
  int *column_map;

  int n_subpops;
  char **subpop_names;

  int n_chm;
  chm_t *chm;
} vcf_t;

enum { VCF_SNP = 0x1, VCF_INSERTION = 0x2, VCF_DELETION = 0x4 };

enum { VCF_LOAD_NOTHING = 0, VCF_LOAD_SNPS_ONLY, VCF_LOAD_INDEL_ONLY, 
       VCF_LOAD_SNPS_AND_INDELS };

enum { GT_AA = 0, GT_AB, GT_BB, GT_NOCALL };

vcf_t *load_genotypes_vcf(char *vcf_fname, 
			  char **include_samples, int n_include,
			  char **exclude_samples, int n_exclude,
			  int load_type, double minimum_maf, double minimum_obs,
			  int snp_qthreshold, int gt_qthreshold);

void vcf_free(vcf_t *vcf);

#endif
